package org.litesoft.orsup.selection;

import org.litesoft.util.IllegalArgument;
import java.util.*;

// Copyright Status:
//
// All Software available from LiteSoft.org (including this file) is
// hereby released into the public domain.
//
// It is free!  As in, you may use it freely in both commercial and
// non-commercial applications, bundle it with your software
// distribution, include it on a CD-ROM, list the source code in a book,
// mirror the documentation at your own web site, or use it in any other
// way you see fit.
//
// NO Warranty!
//
// All software is provided "as is".
//
// There is ABSOLUTELY NO WARRANTY OF ANY KIND: not for the design, fitness
// (for a particular purpose), level of errors (or lack thereof), or
// applicability of this software.  The entire risk as to the quality
// and performance of this software is with you.  Should this software
// prove defective, you assume the cost of all necessary servicing, repair
// or correction.
//
// In no event unless required by applicable law or agreed to in writing
// will any party who created or may modify and/or redistribute this
// software, be liable to you for damages, including any general,
// special, incidental or consequential damages arising out of the use or
// inability to use this software (including but not limited to loss of
// data or data being rendered inaccurate or losses sustained by you or
// third parties or a failure of this software to operate with any
// other programs), even if such holder or other party has been advised
// of the possibility of such damages.
//
// NOTE: Should you discover a bug, have a recogmendation for a change, wish
// to submit modifications, or wish to add new classes/functionality,
// please email them to:
//
//        changes@litesoft.org
//

/**
 * An abstract representation of an associative list of SQL <i>WHERE</i> clauses.<p>
 *
 * This form of <b>WhereClause</b> is primarily used with a <i>list</i> of
 * <b>WhereClause</b>s and something to tie/relate these <b>WhereClause</b>s
 * togther, such as AND or OR.<p>
 *
 * See <a href="WhereClause.html">WhereClause</a><br>
 * See <a href="WhereClauseFactory.html">WhereClauseFactory</a><p>
 *
 * Exceptions: All problems caught when the parameter(s) are checked (as
 * indicated/implied in the @param tags) will generate an IllegalArgumentException,
 * and means the API user has a problem.  If a NullPointerException (or some
 * others, like: ClassCastException or ArrayIndexOutOfBoundsException) is thrown,
 * it means the API developer has a problem.  Any Exception that is explicitly
 * thrown in the API, but unrelated to a parameter, will be listed in the throws
 * clause (and hopefully in the tag @throws).  These may (but probably won't) be
 * checked Exceptions.
 *
 * @author  George Smith
 * @version 1.03 02/07/02 Exception Policy, Use of IllegalArgument class.
 * @version 1.02 11/05/01 Completed? JavaDocs
 * @version 1.01 10/10/01
 * @version 1.0 10/07/01
 */

public abstract class AbstractWhereClauseAssociativeList extends WhereClause
{
    private List zWhereClauseList;
    private String zRelationToString;
    private String zRelationToSql;

    /**
     * Constructor that associates two <b>WhereClause</b>s with the appropriate
     * relationship.<p>
     *
     * Note: If either (or both) of the two <b>WhereClause</b>s are them selves
     * instances of this class AND it (they) has the same relationship, then
     * the <b>WhereClause</b>(s) are <i>merged</i> (collapsed to this level)
     * with this.<p>
     *
     * @param   pType               the Type (AND or OR) (also know as the relationship) for this WhereClause.
     * @param   pRelationToString   String that is used to represent the relationship in the toString() method (!null).
     * @param   pRelationToSql      String that is used to represent the relationship in the toSQL() method (!null).
     * @param   pWhereClause1       1st WhereClause of the two to relate (!null).
     * @param   pWhereClause2       2nd WhereClause of the two to relate (!null).
     */
    protected AbstractWhereClauseAssociativeList( int pType ,
                                                  String pRelationToString ,
                                                  String pRelationToSql ,
                                                  WhereClause pWhereClause1 ,
                                                  WhereClause pWhereClause2 )
    {
        super( pType );

        IllegalArgument.ifNull( "To String Relation" , zRelationToString = pRelationToString);
        IllegalArgument.ifNull( "To SQL Relation" , zRelationToSql = pRelationToSql);

        List list = new ArrayList();
        addWhereClause( list , pWhereClause1 );
        addWhereClause( list , pWhereClause2 );
        zWhereClauseList = Collections.unmodifiableList( list );
    }

    private void addWhereClause( List pList , WhereClause pWhereClause )
    {
        if ( pWhereClause.getType() == this.getType() )
            pList.addAll( ((AbstractWhereClauseAssociativeList) pWhereClause).zWhereClauseList );
        else
            pList.add( pWhereClause );
    }

    /**
     * Accessor for the contained <b>WhereClause</b>s.<p>
     *
     * Note: There will NEVER be less than two, but because of potential
     * merging there might be more than two.<p>
     *
     * @return iteration of contained <b>WhereClause</b>s.
     */
    public final Iterator getWhereClauses()
    {
        return zWhereClauseList.iterator();
    }

    /**
     * Helper method for <b>toString()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see WhereClause#toString()
     */
    protected final void toStringHelper( StringBuffer pSB )
    {
        Iterator it = getWhereClauses();

        toStringHelperParenthesizer( pSB , (WhereClause) it.next() );
        do
        {
            pSB.append( ' ' );
            toStringHelperParenthesizer( pSB , zRelationToString , (WhereClause) it.next() );
        }
        while ( it.hasNext() );
    }

    /**
     * Helper method for <b>toSQL()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * Note: May need to override to get proper result for SQL.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see WhereClause#toSQL()
     */
    protected void toSqlHelper( StringBuffer pSB )
    {
        Iterator it = getWhereClauses();

        toSqlHelperParenthesizer( pSB , (WhereClause) it.next() );
        do
        {
            pSB.append( ' ' );
            toSqlHelperParenthesizer( pSB , zRelationToSql , (WhereClause) it.next() );
        }
        while ( it.hasNext() );
    }
}
